// Copyright 2019 Alexey Krivonogov. All rights reserved.
// Use of this source code is governed by a MIT license
// that can be found in the LICENSE file.

package stdlibvm

import "github.com/gentee/gentee/core"

var Embedded = []core.Embed{
	{Func: StrºBool, Return: core.TYPESTR, Params: []uint16{core.TYPEBOOL}},
	{Func: StrºInt, Return: core.TYPESTR, Params: []uint16{core.TYPEINT}},
	{Func: intºStr, Return: core.TYPEINT, Params: []uint16{core.TYPESTR}, CanError: true},
	{Func: boolºInt, Return: core.TYPEBOOL, Params: []uint16{core.TYPEINT}},
	{Func: ExpStrºInt, Return: core.TYPESTR, Params: []uint16{core.TYPESTR, core.TYPEINT}},
	{Func: ExpStrºBool, Return: core.TYPESTR, Params: []uint16{core.TYPESTR, core.TYPEBOOL}},
	{Func: Command, Params: []uint16{core.TYPESTR}, CanError: true},
	{Func: CommandOutput, Return: core.TYPESTR, Params: []uint16{core.TYPESTR}, CanError: true},
	{Func: GetEnv, Return: core.TYPESTR, Params: []uint16{core.TYPESTR}},
	{Func: SetEnv, Return: core.TYPESTR, Params: []uint16{core.TYPESTR, core.TYPESTR},
		CanError: true},
	// 10
	{Func: SetEnv, Return: core.TYPESTR, Params: []uint16{core.TYPESTR, core.TYPEINT},
		CanError: true},
	{Func: SetEnvBool, Return: core.TYPESTR, Params: []uint16{core.TYPESTR, core.TYPEBOOL},
		CanError: true},
	{Func: AddºCharChar, Return: core.TYPESTR, Params: []uint16{core.TYPECHAR, core.TYPECHAR}},
	{Func: AddºCharStr, Return: core.TYPESTR, Params: []uint16{core.TYPECHAR, core.TYPESTR}},
	{Func: AddºStrChar, Return: core.TYPESTR, Params: []uint16{core.TYPESTR, core.TYPECHAR}},
	{Func: ExpStrºChar, Return: core.TYPESTR, Params: []uint16{core.TYPESTR, core.TYPECHAR}},
	{Func: GreaterºCharChar, Return: core.TYPEBOOL, Params: []uint16{core.TYPECHAR, core.TYPECHAR}},
	{Func: LessºCharChar, Return: core.TYPEBOOL, Params: []uint16{core.TYPECHAR, core.TYPECHAR}},
	{Func: strºChar, Return: core.TYPESTR, Params: []uint16{core.TYPECHAR}},
	{Func: floatºInt, Return: core.TYPEFLOAT, Params: []uint16{core.TYPEINT}},
	// 20
	{Func: floatºStr, Return: core.TYPEFLOAT, Params: []uint16{core.TYPESTR}, CanError: true},
	{Func: AddºFloatInt, Return: core.TYPEFLOAT, Params: []uint16{core.TYPEFLOAT, core.TYPEINT}},
	{Func: AddºIntFloat, Return: core.TYPEFLOAT, Params: []uint16{core.TYPEINT, core.TYPEFLOAT}},
	{Func: intºFloat, Return: core.TYPEINT, Params: []uint16{core.TYPEFLOAT}},
	{Func: StrºFloat, Return: core.TYPESTR, Params: []uint16{core.TYPEFLOAT}},
	{Func: MulºFloatInt, Return: core.TYPEFLOAT, Params: []uint16{core.TYPEFLOAT, core.TYPEINT}},
	{Func: SubºFloatInt, Return: core.TYPEFLOAT, Params: []uint16{core.TYPEFLOAT, core.TYPEINT}},
	{Func: SubºIntFloat, Return: core.TYPEFLOAT, Params: []uint16{core.TYPEINT, core.TYPEFLOAT}},
	{Func: DivºFloatInt, Return: core.TYPEFLOAT, Params: []uint16{core.TYPEFLOAT, core.TYPEINT},
		CanError: true},
	{Func: DivºIntFloat, Return: core.TYPEFLOAT, Params: []uint16{core.TYPEINT, core.TYPEFLOAT},
		CanError: true},
	// 30
	{Func: MulºIntFloat, Return: core.TYPEFLOAT, Params: []uint16{core.TYPEINT, core.TYPEFLOAT}},
	{Func: ExpStrºFloat, Return: core.TYPESTR, Params: []uint16{core.TYPESTR, core.TYPEFLOAT}},
	{Func: EqualºFloatInt, Return: core.TYPEBOOL, Params: []uint16{core.TYPEFLOAT, core.TYPEINT}},
	{Func: GreaterºFloatInt, Return: core.TYPEBOOL, Params: []uint16{core.TYPEFLOAT, core.TYPEINT}},
	{Func: LessºFloatInt, Return: core.TYPEBOOL, Params: []uint16{core.TYPEFLOAT, core.TYPEINT}},
	{Func: TrimSpaceºStr, Return: core.TYPESTR, Params: []uint16{core.TYPESTR}},
	{Func: LinesºStr, Return: core.TYPEARR, Params: []uint16{core.TYPESTR}},
	{Func: MaxºIntInt, Return: core.TYPEINT, Params: []uint16{core.TYPEINT, core.TYPEINT}},
	{Func: MinºIntInt, Return: core.TYPEINT, Params: []uint16{core.TYPEINT, core.TYPEINT}},
	{Func: FormatºStr, Return: core.TYPESTR, Params: []uint16{core.TYPESTR}, Variadic: true},
	// 40
	{Func: ReadFileºStr, Return: core.TYPESTR, Params: []uint16{core.TYPESTR}, CanError: true},
	{Func: JoinºArrStr, Return: core.TYPESTR, Params: []uint16{core.TYPEARR, core.TYPESTR}},
	{Func: SortºArr, Return: core.TYPEARR, Params: []uint16{core.TYPEARR}},
	{Func: SplitºStrStr, Return: core.TYPEARR, Params: []uint16{core.TYPESTR, core.TYPESTR}},
	{Func: bufºStr, Return: core.TYPEBUF, Params: []uint16{core.TYPESTR}},
	{Func: strºBuf, Return: core.TYPESTR, Params: []uint16{core.TYPEBUF}},
	{Func: HexºBuf, Return: core.TYPESTR, Params: []uint16{core.TYPEBUF}},
	{Func: InsertºBufIntBuf, Return: core.TYPEBUF, Params: []uint16{core.TYPEBUF, core.TYPEINT,
		core.TYPEBUF}, CanError: true},
	{Func: UnBase64ºStr, Return: core.TYPEBUF, Params: []uint16{core.TYPESTR}, CanError: true},
	{Func: UnHexºStr, Return: core.TYPEBUF, Params: []uint16{core.TYPESTR}, CanError: true},
	// 50
	{Func: AddºBufBuf, Return: core.TYPEBUF, Params: []uint16{core.TYPEBUF, core.TYPEBUF}},
	{Func: Base64ºBuf, Return: core.TYPESTR, Params: []uint16{core.TYPEBUF}},
	{Func: DelºBufIntInt, Return: core.TYPEBUF, Params: []uint16{core.TYPEBUF, core.TYPEINT,
		core.TYPEINT}, CanError: true},
	{Func: sysBufNil, Return: core.TYPEBUF},
	{Func: Print, Return: core.TYPEINT, Variadic: true, CanError: true},
	{Func: Println, Return: core.TYPEINT, Variadic: true, CanError: true},
	{Func: PrintShiftºStr, Return: core.TYPEINT, Params: []uint16{core.TYPESTR}, CanError: true},
	{Func: ReadString, Return: core.TYPESTR, Params: []uint16{core.TYPESTR}, CanError: true},
	{Func: Md5ºBuf, Return: core.TYPEBUF, Params: []uint16{core.TYPEBUF}},
	{Func: Md5ºStr, Return: core.TYPEBUF, Params: []uint16{core.TYPESTR}},
	// 60
	{Func: Sha256ºBuf, Return: core.TYPEBUF, Params: []uint16{core.TYPEBUF}},
	{Func: Sha256ºStr, Return: core.TYPEBUF, Params: []uint16{core.TYPESTR}},
	{Func: SubstrºStrIntInt, Return: core.TYPESTR, Params: []uint16{core.TYPESTR,
		core.TYPEINT, core.TYPEINT}, CanError: true},
	{Func: FindRegExpºStrStr, Return: core.TYPEARR, Params: []uint16{core.TYPESTR,
		core.TYPESTR}, CanError: true},
	{Func: MatchºStrStr, Return: core.TYPEBOOL, Params: []uint16{core.TYPESTR,
		core.TYPESTR}, CanError: true},
	{Func: ReplaceRegExpºStrStr, Return: core.TYPESTR, Params: []uint16{core.TYPESTR,
		core.TYPESTR, core.TYPESTR}, CanError: true},
	{Func: setºStr, Return: core.TYPESET, Params: []uint16{core.TYPESTR}, CanError: true},
	{Func: UnSetºSet, Return: core.TYPESET, Params: []uint16{core.TYPESET, core.TYPEINT},
		CanError: true},
	{Func: setºArr, Return: core.TYPESET, Params: []uint16{core.TYPEARR}, CanError: true},
	{Func: AppendFileºStrBuf, Params: []uint16{core.TYPESTR, core.TYPEBUF}, CanError: true},
	// 70
	{Func: AppendFileºStrStr, Params: []uint16{core.TYPESTR, core.TYPESTR}, CanError: true},
	{Func: ChDirºStr, Params: []uint16{core.TYPESTR}, CanError: true},
	{Func: CopyFileºStrStr, Return: core.TYPEINT, Params: []uint16{core.TYPESTR, core.TYPESTR},
		CanError: true},
	{Func: CreateDirºStr, Params: []uint16{core.TYPESTR}, CanError: true},
	{Func: GetCurDir, Return: core.TYPESTR, CanError: true},
	{Func: Md5FileºStr, Return: core.TYPESTR, Params: []uint16{core.TYPESTR}, CanError: true},
	{Func: ReadFileºStrBuf, Return: core.TYPEBUF, Params: []uint16{core.TYPESTR, core.TYPEBUF},
		CanError: true},
	{Func: ReadFileºStrIntInt, Return: core.TYPEBUF, Params: []uint16{core.TYPESTR, core.TYPEINT,
		core.TYPEINT}, CanError: true},
	{Func: RemoveºStr, Params: []uint16{core.TYPESTR}, CanError: true},
	{Func: RemoveDirºStr, Params: []uint16{core.TYPESTR}, CanError: true},
	// 80
	{Func: RenameºStrStr, Params: []uint16{core.TYPESTR, core.TYPESTR}, CanError: true},
	{Func: Sha256FileºStr, Return: core.TYPESTR, Params: []uint16{core.TYPESTR}, CanError: true},
	{Func: TempDir, Return: core.TYPESTR},
	{Func: TempDirºStrStr, Return: core.TYPESTR, Params: []uint16{core.TYPESTR, core.TYPESTR},
		CanError: true},
	{Func: WriteFileºStrBuf, Params: []uint16{core.TYPESTR, core.TYPEBUF}, CanError: true},
	{Func: WriteFileºStrStr, Params: []uint16{core.TYPESTR, core.TYPESTR}, CanError: true},
	{Func: HasPrefixºStrStr, Return: core.TYPEBOOL, Params: []uint16{core.TYPESTR, core.TYPESTR}},
	{Func: HasSuffixºStrStr, Return: core.TYPEBOOL, Params: []uint16{core.TYPESTR, core.TYPESTR}},
	{Func: boolºFloat, Return: core.TYPEBOOL, Params: []uint16{core.TYPEFLOAT}},
	{Func: CeilºFloat, Return: core.TYPEINT, Params: []uint16{core.TYPEFLOAT}},
	// 90
	{Func: FloorºFloat, Return: core.TYPEINT, Params: []uint16{core.TYPEFLOAT}},
	{Func: MaxºFloatFloat, Return: core.TYPEFLOAT, Params: []uint16{core.TYPEFLOAT,
		core.TYPEFLOAT}},
	{Func: MinºFloatFloat, Return: core.TYPEFLOAT, Params: []uint16{core.TYPEFLOAT,
		core.TYPEFLOAT}},
	{Func: RoundºFloat, Return: core.TYPEINT, Params: []uint16{core.TYPEFLOAT}},
	{Func: RoundºFloatInt, Return: core.TYPEFLOAT, Params: []uint16{core.TYPEFLOAT, core.TYPEINT}},
	{Func: DelºMapStrAuto, Return: core.TYPEMAP, Params: []uint16{core.TYPEMAP, core.TYPESTR}},
	{Func: IsKeyºMapStrAuto, Return: core.TYPEBOOL, Params: []uint16{core.TYPEMAP, core.TYPESTR}},
	{Func: AbsPath, Return: core.TYPESTR, Params: []uint16{core.TYPESTR}, CanError: true},
	{Func: BaseName, Return: core.TYPESTR, Params: []uint16{core.TYPESTR}},
	{Func: Dir, Return: core.TYPESTR, Params: []uint16{core.TYPESTR}},
	// 100
	{Func: Ext, Return: core.TYPESTR, Params: []uint16{core.TYPESTR}},
	{Func: JoinPath, Return: core.TYPESTR, Variadic: true},
	{Func: MatchPath, Return: core.TYPEBOOL, Params: []uint16{core.TYPESTR, core.TYPESTR},
		CanError: true},
	{Func: SetºSet, Return: core.TYPESET, Params: []uint16{core.TYPESET, core.TYPEINT},
		CanError: true},
	{Func: ToggleºSetInt, Return: core.TYPEBOOL, Params: []uint16{core.TYPESET, core.TYPEINT},
		CanError: true},
	{Func: strºSet, Return: core.TYPESTR, Params: []uint16{core.TYPESET}},
	{Func: arrºSet, Return: core.TYPEARR, Params: []uint16{core.TYPESET}},
	{Func: TrimRightºStr, Return: core.TYPESTR, Params: []uint16{core.TYPESTR, core.TYPESTR}},
	{Func: UpperºStr, Return: core.TYPESTR, Params: []uint16{core.TYPESTR}},
	{Func: BitAndºSetSet, Return: core.TYPESET, Params: []uint16{core.TYPESET, core.TYPESET}},
	// 110
	{Func: BitOrºSetSet, Return: core.TYPESET, Params: []uint16{core.TYPESET, core.TYPESET}},
	{Func: BitNotºSet, Return: core.TYPESET, Params: []uint16{core.TYPESET}},
	{Func: boolºStr, Return: core.TYPEBOOL, Params: []uint16{core.TYPESTR}},
	{Func: FindºStrStr, Return: core.TYPEINT, Params: []uint16{core.TYPESTR, core.TYPESTR}},
	{Func: LeftºStrInt, Return: core.TYPESTR, Params: []uint16{core.TYPESTR, core.TYPEINT}},
	{Func: LowerºStr, Return: core.TYPESTR, Params: []uint16{core.TYPESTR}},
	{Func: RepeatºStrInt, Return: core.TYPESTR, Params: []uint16{core.TYPESTR, core.TYPEINT}},
	{Func: ReplaceºStrStrStr, Return: core.TYPESTR, Params: []uint16{core.TYPESTR,
		core.TYPESTR, core.TYPESTR}},
	{Func: ShiftºStr, Return: core.TYPESTR, Params: []uint16{core.TYPESTR}},
	{Func: Download, Return: core.TYPEINT, Params: []uint16{core.TYPESTR, core.TYPESTR},
		CanError: true},
	// 120
	{Func: HTTPGet, Return: core.TYPEBUF, Params: []uint16{core.TYPESTR}, CanError: true},
	{Func: HTTPPage, Return: core.TYPESTR, Params: []uint16{core.TYPESTR}, CanError: true},
}

var EmbedInt = []core.AssignIntFunc{
	AssignºIntInt,
	AssignAddºIntInt,
	AssignSubºIntInt,
	AssignMulºIntInt,
	AssignDivºIntInt,
	AssignModºIntInt,
	AssignBitAndºIntInt,
	AssignBitOrºIntInt,
	AssignBitXorºIntInt,
	AssignLShiftºIntInt,
	AssignRShiftºIntInt,
	IncDecºInt,
}

var EmbedFloat = []core.AssignFloatFunc{
	AssignºFloatFloat,
	AssignAddºFloatFloat,
	AssignSubºFloatFloat,
	AssignMulºFloatFloat,
	AssignDivºFloatFloat,
}

var EmbedStr = []core.AssignStrFunc{
	AssignºStrStr,
	AssignAddºStrStr,
	AssignºStrBool,
	AssignºStrInt,
	AssignAddºStrChar,
}

var EmbedAny = []core.AssignAnyFunc{
	AssignºArrArr,
	AssignAddºArrAny,
	AssignºMapMap,
	AssignAddºBufChar,
	AssignAddºBufInt,
	AssignAddºBufBuf,
	AssignAddºBufStr,
	AssignºFnFn,
	AssignAddºSetSet,
}
